import typer
import logging
from typing_extensions import Annotated
from rvn.lib.toolbox import UEOTools, ExifPDFTools
from rvn.core.ueopdf import UEOPDF
from pathlib import Path
from rvn.lib.typerExtend import TyperExtend
from rich.progress import Progress, TextColumn, TimeElapsedColumn, BarColumn


app = typer.Typer()

logger = logging.getLogger(__name__)


def log_callback(value: bool):
    if value:
        logging.getLogger('fpdf').setLevel(logging.ERROR)
        logging.getLogger('fontTools').setLevel(logging.ERROR)
        logging.getLogger('PIL').setLevel(logging.ERROR)
        logging.basicConfig(format='%(asctime)s :: %(name)s :: %(levelname)s :: %(message)s',
                            level=logging.DEBUG,
                            handlers=[logging.StreamHandler()])


@app.command()
def generate(filexlsx: Annotated[str, typer.Argument(help="Fichier excel contenant les données étudiantes")],
             outputdir: Annotated[str, typer.Option(..., "--output", "-o", help="Chemin du dossier de destination")],
             semestre: Annotated[str, typer.Option("--semestre", "-s", help="Semestre de la validation de l'UEO")] = "1",
             annee: Annotated[str, typer.Option("--annee", "-a", help="Annee universitaire de la validation de l'UEO")] = "2023-204"):
    """ Génère les attestations UEO au format pdf dans un dossier spécifique à partir d'un fichier d'un export excel
    (fournis par Danae Delonglee danae.delonglee@univ-reunion.fr)

    Args:
        filexlsx (str): fichier excel comprenant les informations des étudiants ayant validé leur attestation UEO
        outputdir (str): chemin du dossier de destination des fichiers pdf générés
        semestre (str): valeur du semestre pour l'attestation. "1" (premier semestre) par défaut.
        annee (str): annee universitaire de la validation de l'UEO
    """
    datas = UEOTools.data_excel(filexlsx)
    TyperExtend.valid_echo('Chargement du fichier excel')
    with Progress(
        BarColumn(),
        TextColumn("[progress.description]{task.description}"),
        TimeElapsedColumn(),
    ) as progress:
        task = progress.add_task(description="[cyan]Génération des fichiers pdf en cours...",
                                 total=len(datas.root))
        for etudiant in datas.root:
            ueopdf = UEOPDF()
            ueopdf.content(data=etudiant, semestre=semestre, annee_universitaire=annee)
            progress.console.print(f'► Génération du fichier pdf pour l\'étudiant {etudiant.numero_etudiant}')
            ueopdf.output(f"{outputdir}/{etudiant.numero_etudiant}_{etudiant.sk}.pdf")
            progress.update(task, advance=1)

    TyperExtend.valid_echo('Opération réalisé avec succès')
    TyperExtend.info_echo(f'Les pdf ont été sauvegardé dans le dossier "{Path(outputdir).resolve()}"')


@app.command()
def is_original_file(pdffile: Annotated[str, typer.Argument(help="Fichier pdf attestation ueo")],
                     verbose: bool = typer.Option(None, "--verbose", callback=log_callback, is_eager=True)):
    """ Permet de savoir si le fichier pdf ueo est le fichier original généré par l'application ou non

    Args:
        pdffile (str): Chemin du fichier pdf
        verbose (bool): Active ou désactive logger

    Returns:

    """

    if Path(pdffile).exists() and Path(pdffile).suffix == '.pdf':
        if ExifPDFTools(pdffile).is_original_file():
            TyperExtend.valid_echo('Ce fichier est bien l\'originial et a été généré avec la lib **rvn**')
        else:
            TyperExtend.error_echo('Ce fichier n\'a pas été généré par l\'application rvn')
    elif Path(pdffile).suffix != '.pdf':
        TyperExtend.error_echo('Erreur : Le fichier n\'est pas un fichier .pdf')
        TyperExtend.exit_unsuccess()



