from fpdf import FPDF
from fpdf.enums import AccessPermission
from datetime import datetime
from rvn.config.settings import FONTS, IMG, SUJET, AUTEUR
from rvn.model.etudiant import EtudiantForUEO
from rvn.lib.toolbox import XMPMetadata
import base64


class UEOPDF(FPDF):

    def __init__(self):
        super().__init__()
        self.add_font(family="playfairDisplay", fname=f"{FONTS}/playfairDisplay.ttf")
        self.add_font(family="playfairDisplay", style="B", fname=f"{FONTS}/playfairDisplay-bold.ttf")
        self.add_font(family="raleway", fname=f"{FONTS}/raleway.ttf")
        self.add_font(family="raleway", style="B", fname=f"{FONTS}/raleway-bold.ttf")

    def header(self):
        # Rendering logo:
        with self.local_context(fill_opacity=0.5):
            self.image(f"{IMG}/logo_ur.png", 25, 15, 55.6, 12.4, keep_aspect_ratio=True)
        # Performing a line break:
        self.ln(60)

    def footer(self):
        # Position cursor at 1.5 cm from bottom:
        self.set_y(-15)
        # Setting font: helvetica italic 8
        self.set_font("raleway", size=10)
        # Printing label
        with self.local_context(fill_opacity=0.5):
            self.cell(0, 10, "Attestation de réussite UEO", align="L")
            # Printing page number
            self.cell(0, 10, "1/1", align="R")

    def content(self, *, data: EtudiantForUEO, semestre: str, annee_universitaire: str):
        message = (f"               Le Premier vice-Président, Dr Dominique Morau atteste "
                   f"{data.prenom.title()} {data.nom.upper()}, né·e le "
                   f"{data.date_naissance.strftime('%d/%m/%Y')}, inscrit·e en "
                   f"{data.libelle_parcours.upper()} a validé l’Unité d’Enseignement d’Ouverture (UEO) "
                   f"**{data.libelle_ueo}** au semestre {semestre} de l’année universitaire {annee_universitaire}, "
                   f"lui conférant ainsi 2 crédits (ECTS).")

        # self._set_matadata(data)

        def _set_matadata():
            title = (f'Attestation de Réussite UEO S{semestre} -- '
                     f'{data.prenom.title()} {data.nom.upper()} -- '
                     f'{annee_universitaire}')
            keywords = data.dk
            now = datetime.now()

            self.set_title(title)
            self.set_subject(SUJET)
            self.set_author(AUTEUR)
            self.set_creation_date(now)
            self.set_xmp_metadata(XMPMetadata(title=title,
                                              subject=SUJET,
                                              author=AUTEUR,
                                              description=SUJET,
                                              extra=keywords,
                                              create_date=now.strftime('%d/%m/%Y')).generate_xmp_metadata()
                                  )

        def _set_encryption():
            tmp = str(data.numero_etudiant * int(data.notes))
            self.set_encryption(
                owner_password=base64.b64encode(tmp.encode()).decode('utf-8'),
                permissions=AccessPermission.PRINT_LOW_RES |
                            AccessPermission.PRINT_HIGH_RES |
                            AccessPermission.COPY_FOR_ACCESSIBILITY
            )

        _set_matadata()
        _set_encryption()

        self.set_font("playfairDisplay", size=11)
        self.add_page()
        self.set_margin(25)
        self.set_fill_color(213, 220, 228)
        self.cell(0, 11, "ATTESTATION DE RÉUSSITE - UNITÉS D’ENSEIGNEMENT D’OUVERTURE", align="C", fill=True, border=1)
        self.ln(30)

        self.set_font("raleway", size=11)
        self.multi_cell(0, 11, message, align="L", markdown=True)
        self.ln(20)

        self.cell(0, 11, "Fait pour servir et valoir ce que de droit", align="L")
        self.ln(30)

        self.cell(0, 11, datetime.now().strftime('%d/%m/%Y'), align="R")
        self.image(f"{IMG}/signature-deleg-jg.jpg", 75, 230, 108, 35)
