import logging
import pandas as pd
import xml.etree.ElementTree as ET
import exiftool
from rvn.model.etudiant import ListEtudiantForUEO
from rvn.config.settings import PASSPHRASE
from dataclasses import dataclass, field
from pathlib import Path
from itsdangerous.url_safe import URLSafeSerializer
from itsdangerous.exc import BadSignature
from typing import Any

logger = logging.getLogger(__name__)


class UEOTools:

    @staticmethod
    def data_excel(excel_file: str) -> ListEtudiantForUEO:
        """ Récupère les informations du fichier xlsx sous forme de liste

        Args:
            excel_file (str): Chemin du fichier excel à traiter

        Returns:
            Une liste contenant les informations du fichier excel (attribut excel_file de l'objet Publipostage)

        """
        logger.info(f'Chargement du fichier excel {excel_file}')
        data = pd.read_excel(excel_file)
        data['Date de naissance'] = pd.to_datetime(data['Date de naissance'])
        tmp = data.astype({'Individu - Nom': 'string', 'Individu - Prénom': 'string', 'IAE - Etape (lib.)': 'string',
                            'UEO ': 'string', 'Individu - Code Etudiant': pd.Int64Dtype()})

        data = tmp.to_dict(orient='records')
        return ListEtudiantForUEO(data)


@dataclass
class XMPMetadata:
    title: str
    subject: str
    author: str
    description: str
    extra: str
    create_date: str

    def generate_xmp_metadata(self):
        xmp_root = ET.Element("{adobe:ns:meta/}xmpmeta")
        rdf_rdf = ET.SubElement(xmp_root, "{http://www.w3.org/1999/02/22-rdf-syntax-ns#}RDF")
        rdf_description = ET.SubElement(rdf_rdf, "{http://www.w3.org/1999/02/22-rdf-syntax-ns#}Description")

        # DC (Dublin Core) metadata
        dc_ns = "http://purl.org/dc/elements/1.1/"
        ET.SubElement(rdf_description, f"{{{dc_ns}}}title").text = self.title
        ET.SubElement(rdf_description, f"{{{dc_ns}}}creator").text = self.author
        ET.SubElement(rdf_description, f"{{{dc_ns}}}description").text = self.description

        # XMP metadata
        xmp_ns = "http://ns.adobe.com/xap/1.0/"
        ET.SubElement(rdf_description, f"{{{xmp_ns}}}CreateDate").text = self.create_date
        ET.SubElement(rdf_description, f"{{{xmp_ns}}}extra").text = self.extra

        # Convert the XML tree to a string
        xmp_metadata_str = ET.tostring(xmp_root, encoding="utf-8").decode("utf-8")

        return xmp_metadata_str


@dataclass
class ExifPDFTools:
    file: str
    data: Any = field(init=False)

    def is_original_file(self) -> bool:
        if Path(self.file).exists():
            logger.debug(f'Le fichier {self.file} existe')
            with exiftool.ExifToolHelper() as et:
                logger.debug('Extraction des metadata du fichier')
                metadata = et.get_metadata(self.file)
                key = metadata[0]['XMP:Extra']
                s = URLSafeSerializer(PASSPHRASE, salt='ueo')
                try:
                    logger.debug('Clé trouvé')
                    self.data = s.loads(key)
                    return True
                except BadSignature as e:
                    logger.error(f'Error : {e}')
                    return False
        return False

